/*global define*/
define([
    'backbone.marionette', './config', './views/editor', './views/pagelist',
    './views/device', './collections/pages', './collections/widgets', './views/widgetbar',
    './views/username', './models/user', './views/editortools'
], function (Marionette, config, Editor, PageList, Device, PageListCollection, WidgetCollection, Widgetbar, Username, UserModel, Editortools) {

    'use strict';

    var app = new Marionette.Application();

    app.addRegions({
        storefront: '[data-region="content"]',
        editortools: '[data-region="editortools"]',
        contenttools: '[data-region="contenttools"]'
    });

    // Create application wide
    // namespace for models
    app.addInitializer(function () {
        this.models = {};
    });

    // init current user model
    app.addInitializer(function () {
        this.models.user = new UserModel({
            id: config.currentUserId
        });
        this.models.user.url = this.models.user.urlRoot + '/' + config.currentUserId;
        this.models.user.fetch();
    });

    app.addInitializer(function () {
        var editor = new Editor({
            app: app
        });
        // Show `editor` in the storefront.
        this.storefront.show(editor);
    });

    app.addInitializer(function () {
        // Init `editortools`.
        var pagelist = new PageList({
                collection: new PageListCollection(),
                app: app
            }),
            device = new Device(),
            username = new Username({ model: this.models.user }),
            unitytoolbox = new Widgetbar({
                app: this,
                collection: new WidgetCollection([
                    {
                        type: 'image',
                        title: 'Add Image',
                        id: 'add-image',
                        icon: 'picture',
                        description: 'add an image'
                    }, {
                        type: 'text',
                        title: 'Add Text',
                        id: 'add-text',
                        icon: 'font',
                        description: 'add a text'
                    }, {
                        type: 'row',
                        title: 'add Row',
                        id: 'add-row',
                        icon: 'columns',
                        description: 'Add a new row to the page'
                    }
                ])
            }),
            editortools = new Editortools();

        // Show `pagelist` in `editorbar`.
        this.editortools.show(editortools);

        // Init `editortools`.
        editortools.one.show(username);
        editortools.two.show(pagelist);
        editortools.three.show(device);

        // Show `unitytoolbox` in the `unitytoolboxRegion`.
        this.contenttools.show(unitytoolbox);
    });

    return app;
});
