/*global define*/
define([
    'sandbox/event',
    'sandbox/dom',
    'underscore',
    'backbone.marionette',
    'app/templates/views/outline/outline'
], function (vent, $, _, Marionette, template) {

    'use strict';

    var styles = {
            'default': 'outline-default',
            'editable': 'outline-editable',
            'forbidden': 'outline-forbidden',
            'droppable': 'outline-droppable'
        },

        // This way we can clean up all available
        // `styles` with `.removeClass(stylesClasses)`.
        stylesClasses = _.values(styles).join(' '),

        // Align `this.$el` to `lastHovered` with respect to
        // its location inside `this.options.iframe`.
        // Optionally a `style` (see `styles` hash above)
        // will be attached to (`addClass`) `this.$el`.
        // If `actions` is truthy, `this.ui.actionbox` will be shown.
        alignTo = function (lastHovered, style, actions) {
            var that = this,
                $iframe = that.options.iframe,
                iframeBoundingRect = $iframe.get(0).getBoundingClientRect(),
                iframeBodyBoundingRect = $($iframe.get(0).contentDocument).find('body').get(0).getBoundingClientRect(),
                lastHoveredBoundingRect,
                //iframeScrollY = $($('iframe').get(0).contentWindow).scrollTop(),
                top,
                left_correct,
                width_correct,
                height_correct,
                top_correct,
                i = 0,
                isBody;

            if (!lastHovered) {
                that.hide();
                that.lastHovered = null;
                return;
            }

            isBody = 'body' === lastHovered.nodeName.toLowerCase();
            lastHoveredBoundingRect = isBody ? _.clone(iframeBoundingRect) : lastHovered.getBoundingClientRect();

            if (isBody) {
                lastHoveredBoundingRect.right = lastHoveredBoundingRect.right - lastHoveredBoundingRect.left;
                lastHoveredBoundingRect.left = 0;
                lastHoveredBoundingRect.bottom = lastHoveredBoundingRect.bottom - lastHoveredBoundingRect.top;
                lastHoveredBoundingRect.top = 0;
            }

            top = lastHoveredBoundingRect.top; //- iframeScrollY;
            left_correct = -Math.min(lastHoveredBoundingRect.left, i);
            top_correct = -Math.min(Math.max(lastHoveredBoundingRect.top, 0), i);
            width_correct = Math.min(Math.abs(iframeBodyBoundingRect.width - lastHoveredBoundingRect.right), i) - left_correct;
            height_correct = Math.min(Math.abs(iframeBoundingRect.bottom - lastHoveredBoundingRect.bottom), i) - top_correct;

            that.$el
                .removeClass(stylesClasses)
                .addClass(styles[style] || styles['default'])
                .css({
                    left: lastHoveredBoundingRect.left + left_correct,
                    top: top + top_correct,
                    width: lastHoveredBoundingRect.width + width_correct,
                    height: lastHoveredBoundingRect.height + height_correct,
                    opacity: 1
                });

            if (actions) {
                this.showActionbox();
            } else {
                this.hideActionbox();
            }

            that.lastHovered = lastHovered;

        };

    return Marionette.ItemView.extend({

        options: {
            // Reference to the `iframe`
            // where the elements
            // we want to highlight are
            // located.
            iframe: null
        },

        attributes: {'data-view': 'outline'},
        className: 'outline outline-guide',
        template: template,

        ui: {
            actionbox: '.editor-element-toolbox'
        },

        events: {
            'click .tool-remove': 'deleteContent'
        },

        initialize: function () {
            this.listenTo(vent, 'hovered.last', alignTo);
            // this.listenTo(vent, mouse.highlightEvent, alignTo);
            // this.listenTo(vent, 'outline.actions:show', this.showActionbox);
            // this.listenTo(vent, 'outline.actions:hide', this.hideActionbox);
        },

        // Trigger `remove` action on
        // `this.lasthovered`.
        deleteContent: function (e) {
            vent.trigger('editor.content:remove', this.lastHovered);
            e.preventDefault();
        },

        // Hide `this.$el` and
        // `this.ui.actionbox`.
        hide: function () {
            this.hideActionbox();
            this.$el.css('opacity', 0);
        },

        // Show `this.ui.actionbox`.
        showActionbox: function () {
            if (!this.$el.hasClass(styles.droppable) && !this.$el.hasClass(styles.forbidden)) {
                this.ui.actionbox.show();
            }
        },

        // Hide `this.ui.actionbox`.
        hideActionbox: function () {
            this.ui.actionbox.hide();
        }

    });

});
