/*global define*/
define([
    'backbone.marionette',
    '../config',
    'sandbox/event',
    '../views/pagelistitem',
    '../templates/views/pagelist/pagelist',
    '../templates/includes/button',
    'handlebars'
], function (Marionette, config, vent, PageView, template, button_tmpl, Handlebars) {

    'use strict';

    return Marionette.CompositeView.extend({

        template: template,

        itemView: PageView,

        attributes: {
            'data-view': 'pagelist'
        },

        itemViewContainer: 'ul',

        events: {
            'click #add-page': 'addPage'
        },

        collectionEvents: {
            'sync': 'render'
        },

        itemViewOptions: function () {
            return {
                app: this.options.app
            };
        },

        initialize: function () {
            this.collection.fetch();
            Handlebars.registerPartial('includes/button', button_tmpl);
        },

        serializeData: function () {
            return {
                hasItems: (this.collection && this.collection.length > 0) ? true : false,
                add_page_button: {
                    id: 'add-page',
                    text: 'Add page'
                }
            };
        },

        addPage: function () {
            /*global prompt*/
            this.createPage(prompt('Page name', 'Page (' + (this.collection.length + 1) + ')'));

        },

        createPage: function (pageName) {
            if (!pageName) {
                return;
            }
            this.collection.create({
                name: pageName,
                shopGuid: config.shopGuid
            }, {
                wait: true,
                success: function (model) {
                    vent.trigger('editor.view:show', model);
                }
            });
        }

    });

});
