/*global define, confirm*/
define([
    'sandbox/event',
    'backbone.marionette',
    '../templates/views/pagelistitem/pagelistitem'
], function (vent, Marionette, template) {

    'use strict';

    return Marionette.ItemView.extend({

        tagName: 'li',

        template: template,

        attributes: {
            'data-view': 'pagelistitem'
        },

        events: {
            'click .open-page': 'openPage',
            'click .edit-trigger': 'changeName',
            'click .remove-trigger': 'removePage',
            'click .visibility-trigger': 'switchVisibility'
        },

        modelEvents: {
            'change': 'render'
        },

        switchVisibility: function () {
            this.model.save('visible', !(this.model.get('visible')));
        },

        openPage: function () {

            vent.trigger('editor.view:show', this.model);

        },

        removePage: function () {
            //
            // TODO
            // Needs to be moved to some
            // »deleteHandler« Object.
            //
            var self = this,
                destroyModel = function () {
                    self.model.destroy({
                        wait: true,
                        success: function (model) {
                            vent.trigger('editor.view:close', model);
                        }
                    });
                };

            if (this.options.app.models.user.get('deleteConfirmation')) {
                if (confirm('Are you sure you want to do delete the page?')) {
                    destroyModel();
                }
            } else {
                destroyModel();
            }

        },

        changeName: function () {
            var self = this;
            Marionette.$('<input type="text" id="page-name" />')
                .on('blur keypress', function (event) {
                    if (event.type === 'keypress' && event.which !== 13) {
                        return;
                    }
                    if (this.value === self.model.get('name')) {
                        self.render();
                        return;
                    }
                    self.model.set('name', this.value);
                    self.model.save();
                })
                .replaceAll(this.$('.open-page'))
                .trigger('focus')
                .val(this.model.get('name'));
        }

    });

});
