/*global define, describe, beforeEach, it, expect, jasmine*/
define(['app/models/column', 'app/collections/rows'], function (Column, Rows) {

    'use strict';

    return describe('Model :: Column', function () {

        beforeEach(function () {
            this.column = new Column();
        });

        describe('.initialize()', function () {
            it('should create a *default* row', function () {
                expect(this.column).not.toBe(null);
                expect(this.column.get('slots')).toEqual(12);
                expect(this.column.get('offset')).toEqual(0);
                expect(this.column.get('rows')).toEqual(jasmine.any(Rows));
                expect(this.column.get('rows').length).toBe(0);
                expect(this.column.get('content')).toBe(null);
            });
            it('should return a vaild JSON with empty rows', function () {
                var json = this.column.toJSON();
                expect(json.slots).toBe(12);
                expect(json.offset).toBe(0);
                expect(json.rows).not.toBe(null);
                expect(json.rows.length).toEqual(0);
                expect(json.content).toBe(null);
            });
        });

        describe('adding rows ', function () {
            it('via .set() should work', function () {
                this.column.set({ rows: [{}] });
                expect(this.column.get('rows')).toEqual(jasmine.any(Rows));
                expect(this.column.get('rows').length).toBe(1);
            });
            it('via .get() and using the add function of the collection should work', function () {
                this.column.get('rows').add({});
                expect(this.column.get('rows')).toEqual(jasmine.any(Rows));
                expect(this.column.get('rows').length).toBe(1);
            });
        });

        describe('validation: ', function () {
            it('adding content to a column with rows should cause a validationError', function () {
                this.column.set({rows: [{}]});
                this.column.set({content: {}}, {validate: true});
                expect(this.column.get('rows').length).toBe(1);
                expect(this.column.get('content')).toBeNull();
                expect(this.column.validationError).toBe('a column with content may not have rows');
            });
            it('adding a row to a column with content should cause a validationError', function () {
                this.column.set({content: {}});
                this.column.set({rows: [{}]}, {validate: true});
                expect(this.column.get('content')).not.toBeNull();
                expect(this.column.get('rows').length).toBe(0);
                expect(this.column.validationError).toBe('a column with content may not have rows');
            });
        });

        describe('toJSON()', function () {
            it('should return a json of the column with the rows', function () {
                this.column.get('rows').add({});
                var json = this.column.toJSON();
                expect(json.rows).toEqual(jasmine.any(Array));
            });
        });

    });

});
