/*global define, describe, beforeEach, it, expect, jasmine*/
define(['app/models/page', 'app/collections/rows'], function (Page, RowCollection) {

    'use strict';

    describe('Model :: Page', function () {

        beforeEach(function () {
            this.addMatchers({
                toHaveNumElements: function (expected) {
                    return this.actual.length === expected;
                }
            });

            this.page = new Page();
            this.page.collection = {
                url: '/rs/page'
            };
        });

        describe('initialize()', function () {
            it('should have default values', function () {
                expect(this.page.get('name')).toEqual('Page');
                expect(this.page.get('rows')).toEqual(jasmine.any(RowCollection));
            });
        });

        describe('url', function () {
            describe('when no id is set', function () {
                it('should return the collection URL', function () {
                    expect(this.page.url()).toEqual('/rs/page');
                });
            });
            describe('when id is set', function () {
                it('should return the collection URL and id', function () {
                    this.page.id = 1;
                    expect(this.page.url()).toEqual('/rs/page/1');
                });
            });
        });

        describe('addContent()', function () {
            it('should create a new row with one column containing the content', function () {
                var content = {};
                expect(this.page.get('rows')).toHaveNumElements(0);
                this.page.addContent(content);
                expect(this.page.get('rows')).toHaveNumElements(1);
                expect(this.page.get('rows').at(0).get('columns').at(0).get('content')).toEqual(content);
            });
        });

        describe('toJSON()', function () {
            it('should return a json of the page with the rows', function () {
                this.page.get('rows').add({});
                var json = this.page.toJSON();
                expect(json.rows).toEqual(jasmine.any(Array));
            });
        });

    });
});
