/*global define, describe, beforeEach, it, runs, expect, jasmine*/
define(['app/models/row', 'app/collections/columns'], function (Row, Columns) {

    'use strict';

    return describe('Model :: Row', function () {

        beforeEach(function () {
            this.row = new Row();
        });

        describe('.initialize()', function () {

            it('should create a *default* row', function () {

                runs(function () {
                    expect(this.row).not.toBe(null);
                    expect(this.row.get('columns')).toEqual(jasmine.any(Columns));
                });

            });

            it('should return a vaild JSON with empty columns', function () {
                var json = this.row.toJSON();
                runs(function () {
                    expect(json.columns).not.toBe(null);
                    expect(json.columns.length).toEqual(0);
                });
            });

        });

        describe('.set()', function () {

            it('should create two columns', function () {

                this.row.set({ columns: [{slots: 6}, {slots: 6}] });

                runs(function () {
                    expect(this.row.get('columns')).toEqual(jasmine.any(Columns));
                    expect(this.row.get('columns').length).toBe(2);
                });

            });

        });

        describe('adding columns', function () {

            it('should add an empty column', function () {

                this.row.get('columns').add({});

                runs(function () {
                    expect(this.row.get('columns').length).toBe(1);
                });

            });

        });

        describe('toJSON()', function () {
            it('should return a json of the row with the columns', function () {
                this.row.get('columns').add({});
                var json = this.row.toJSON();
                expect(json.columns).toEqual(jasmine.any(Array));
            });
        });

    });

});
