/*global define, describe, beforeEach, afterEach, it, expect, sinon*/
define(['app/views/device', 'sandbox/event', 'app/constants/device'], function (DeviceView, vent, device) {

    'use strict';

    describe('View :: Device', function () {

        beforeEach(function () {
            this.view = new DeviceView();
        });

        afterEach(function () {
            this.view.close();
        });

        describe('.initialize()', function () {

            it('should have a default DIV element', function () {

                expect(this.view.$el).toBe('div');

            });

        });

        describe('.render()', function () {

            it('should render three device buttons', function () {

                this.view.render();

                expect(this.view.$el.find('button').length).toBe(3);

            });

        });

        describe('events', function () {

            beforeEach(function () {
                this.view.render();

                this.spy = sinon.spy();
                vent.on('device', this.spy);
            });

            afterEach(function () {
                this.spy = null;
                vent.off('device', this.spy);
            });

            it('Clicking button#go-mobile should trigger a »device« event', function () {

                this.view.$el.find('button#go-mobile').trigger('click');
                this.view.$el.find('button#go-mobile').trigger('click');

                expect(this.spy).toHaveBeenCalledOnce();
                expect(this.spy).toHaveBeenCalledWithExactly(device.MOBILE);

            });

            it('Clicking button#go-tablet should trigger a »device« event', function () {

                this.view.$el.find('button#go-tablet').trigger('click');
                this.view.$el.find('button#go-tablet').trigger('click');

                expect(this.spy).toHaveBeenCalledOnce();
                expect(this.spy).toHaveBeenCalledWithExactly(device.TABLET);


            });

            it('Clicking button#go-laptop should not trigger a "device" event, since laptop is already selected', function () {

                this.view.$el.find('button#go-laptop').trigger('click');

                expect(this.spy).not.toHaveBeenCalled();

            });

            it('Clicking button#go-laptop should trigger a "device" event after another device has been selected', function () {

                this.view.$el.find('button#go-tablet').trigger('click');
                this.view.$el.find('button#go-laptop').trigger('click');

                expect(this.spy).toHaveBeenCalledTwice();
                expect(this.spy).toHaveBeenCalledWithExactly(device.LAPTOP);
            });

        });

    });

});
