/*global define, describe, beforeEach, afterEach, waitsFor, it, runs, waitsFor, expect, sinon, window*/
define(['sandbox/event', 'app/views/pagelist', 'app/collections/pages'], function (vent, PageList, Pages) {

    'use strict';

    describe('views :: pagelist', function () {

        beforeEach(function () {
            this.collection = new Pages();
            this.view = new PageList({collection: this.collection});
        });

        afterEach(function () {
            this.view.close();
        });

        describe('.initialize()', function () {
            it('should create a div', function () {
                runs(function () {
                    expect(this.view.el.nodeName).toEqual('DIV');
                });
            });
        });

        describe('Rendering', function () {
            it('should render an empty list without pages', function () {
                this.view.render();
                runs(function () {
                    expect(this.view.$el).toContain('ul');
                    expect(this.view.$el.find('li')).toBeEmpty();
                });
            });

            it('should render a filled list with pages', function () {
                this.view.render();
                this.collection.add({id: '1', name: 'Testpage'});
                runs(function () {
                    expect(this.view.$el).toContain('ul');
                    expect(this.view.$el.find('li')).toHaveLength(2);
                    expect(this.view.$el.find('li').get(1)).toContainText('Testpage');
                    expect(this.view.$el.find('li').get(1)).toContain('[data-id="1"]');
                });
            });
        });

        describe('Creation', function () {
            var returned;

            beforeEach(function () {
                this.server = sinon.fakeServer.create();
                this.promptStub = sinon.stub(window, 'prompt', function () {
                    return returned;
                });
                this.collectionCreateSpy = sinon.spy(this.collection, 'create');
                this.triggerSpy = sinon.spy();
                vent.on('editor.view:show', this.triggerSpy);
                this.view.render();
            });

            afterEach(function () {
                this.server.restore();
                this.promptStub.restore();
                this.collectionCreateSpy.restore();
            });

            it('should *not* create an new page', function () {
                this.view.createPage();
                expect(this.collection.length).toBe(0);
            });

            it('should create a new page', function () {
                returned = 'Page (1)';

                this.server.respondWith('POST', 'page', [200, {
                    'Content-Type': 'application/json'
                }, JSON.stringify({
                    name: returned,
                    rows: []
                })]);

                // `addPage` should call `collection.create`
                runs(function () {
                    this.view.addPage();
                    this.server.respond();
                    expect(this.collectionCreateSpy).toHaveBeenCalled();
                });

                // wait until the new page is inserted into the collection
                waitsFor(function () {
                    return this.collection.length === 1;
                }, '', 750);

                // run the next expectations
                runs(function () {
                    expect(this.collection.length).toBe(1);
                    expect(this.collection.at(0).get('name')).toBe(returned);
                    expect(this.triggerSpy).toHaveBeenCalledOnce();
                });
            });
        });

    });
});

