/*global define, describe, beforeEach, afterEach, it, expect, sinon, window, $, spyOn, jasmine*/
define(['sandbox/event', 'app/views/row'], function (vent, RowButton) {

    'use strict';

    describe('Views :: RowButton.', function () {

        beforeEach(function () {
            this.model = {
                toJSON: function () {
                    return {
                        type: 'row',
                        title: 'Add Row',
                        id: 'add-row',
                        icon: 'columns',
                        description: 'add a new row tot he page'
                    };
                }
            };
            this.view = new RowButton({
                model: this.model
            });
            this.view.render();
        });

        afterEach(function () {
            this.view.close();
        });

        describe('action', function () {

            beforeEach(function () {
                this.eventSpy = sinon.spy();
                vent.on('editor.row:add', this.eventSpy);
            });

            afterEach(function () {
                this.eventSpy.reset();
            });

            it('must trigger the add row event if a valid number of columns is entered', function () {
                spyOn(window, 'prompt').andReturn('2');

                this.view.action();

                expect(this.eventSpy).toHaveBeenCalledOnce();
                expect(this.eventSpy.args[0][0].columns.length).toBe(2);
                expect(this.eventSpy.args[0][0].columns[0].slots).toBe(6);
                expect(this.eventSpy.args[0][0].columns[1].slots).toBe(6);
            });

            it('must not trigger the add row event if an invalid number of columns is entered', function () {
                spyOn(window, 'prompt').andReturn('5');

                this.view.action();

                expect(this.eventSpy).not.toHaveBeenCalled();
            });

        });

        describe('implements a maybedropped method', function () {
            var $event;

            it('which should exist', function () {
                expect(this.view.maybedropped).toEqual(jasmine.any(Function));
            });

            it('which returns true on an html tag', function () {
                $event = $.Event('dragover', {
                    target: $('<html></html>').get(0)
                });
                expect(this.view.maybedropped($event)).toBe(true);
            });

            it('mostly returns false', function () {
                $event = $.Event('dragover', {
                    target: $('<div data-type="image"></div>').get(0)
                });
                expect(this.view.maybedropped($event)).toBe(false);
            });
        });

    });
});
