/*global define, describe, beforeEach, afterEach, it, expect, sinon, window, $, jasmine*/
define(['app/views/text'], function (TextButton) {

    'use strict';

    describe('Views :: TextButton.', function () {

        beforeEach(function () {
            this.model = {
                toJSON: function () {
                    return {
                        type: 'text',
                        title: 'Add Text',
                        id: 'add-text',
                        icon: 'font',
                        description: 'add a text'
                    };
                }
            };
            this.view = new TextButton({
                model: this.model
            });
            this.view.render();
        });

        afterEach(function () {
            this.view.close();
        });

        describe('action', function () {
            var returned;

            beforeEach(function () {
                this.promptStub = sinon.stub(window, 'prompt', function () {
                    return returned;
                });
                this.addContentStub = sinon.stub(this.view, 'addContent');
            });

            afterEach(function () {
                this.promptStub.restore();
                this.addContentStub.restore();
            });

            it('should add the correct content', function () {
                returned = 'test';
                this.view.action();
                expect(this.promptStub).toHaveBeenCalledOnce();
                expect(this.promptStub).toHaveReturned(returned);
                expect(this.addContentStub).toHaveBeenCalledOnce();
            });

            it('should cancel the call', function () {
                returned = '';
                this.view.action();
                expect(this.promptStub).toHaveBeenCalledOnce();
                expect(this.promptStub).toHaveReturned(returned);
                expect(this.addContentStub).not.toHaveBeenCalled();
            });

        });


        describe('implements a maybedropped method', function () {
            var $event;

            it('which should exist', function () {
                expect(this.view.maybedropped).toEqual(jasmine.any(Function));
            });

            it('which sometimes returns true', function () {
                $event = $.Event('dragover', {
                    target: $('<div data-view="column"></div>').get(0)
                });
                expect(this.view.maybedropped($event)).toBe(true);
            });

            it('which sometimes returns false', function () {
                $event = $.Event('dragover', {
                    target: $('<div data-type="image"></div>').get(0)
                });
                expect(this.view.maybedropped($event)).toBe(false);
            });
        });

    });
});
