/*global define, describe, beforeEach, afterEach, it, expect, sinon, $, jasmine, spyOn*/
define(['app/app', 'sandbox/event', 'app/views/widget'], function (app, vent, Widgetitem) {

    'use strict';

    describe('Views :: Widget', function () {
        var $event;

        beforeEach(function () {
            var that = this;
            this.model = {
                toJSON: function () {
                    return {
                        type: 'test',
                        title: 'test widget',
                        id: 'add-test',
                        icon: 'test',
                        description: 'this is a test'
                    };
                }
            };
            this.lastDropped = $('<span>').data('id', '1');
            this.view = new Widgetitem({ model: this.model, app: app });
            this.view.render();
            this.$widget = this.view.$el.find('span');
            app.editor = {
                pageModel: {
                    id: '1',
                    addContent: function () { return; }
                },
                getLastDropped: function () {
                    return that.lastDropped;
                },
                savePage: sinon.spy()
            };
        });

        afterEach(function () {
            this.view.close();
        });

        it('Rendering should create a "span"', function () {
            expect(this.view.$el).toBe('li');
            expect(this.view.$el).toContain('span');
            expect(this.view.$el.find('span')).toBe(this.$widget);
            expect(this.$widget).toBeMatchedBy('#add-test');
            expect(this.$widget).toHaveAttr('title', 'this is a test');
            expect(this.$widget).toContain('.icon-test');
        });

        it('should implement an onDrop method which calls preventDefault on the given event', function () {
            $event = $.Event('drop');
            spyOn($event, 'preventDefault');
            expect(this.view.onDrop).toEqual(jasmine.any(Function));
            this.view.onDrop($event);

            // Otherwise Firefox goes to hell, after `drop`.
            expect($event.preventDefault).toHaveBeenCalled();
        });

        describe('Add Content', function () {

            beforeEach(function () {
                this.requestData = {};
                this.addContentSpy = sinon.spy();
            });

            afterEach(function () {
                this.addContentSpy = null;
            });

            it('should trigger an add content event', function () {
                vent.on('editor.content:add', this.addContentSpy);

                this.view.addContent(this.requestData);

                expect(this.addContentSpy).toHaveBeenCalledOnce();
                expect(this.addContentSpy).toHaveBeenCalledWith(this.requestData);
            });

        });

        describe('Events', function () {
            beforeEach(function () {
                this.event = $.Event('dragstart');
                this.event.originalEvent = {
                    dataTransfer: {
                        setData: sinon.spy()
                    }
                };
                this.dragStartSpy = sinon.spy(this.view, 'dragStart');
                this.setDataSpy = this.event.originalEvent.dataTransfer.setData;
                spyOn(vent, 'trigger');
            });

            afterEach(function () {
                this.dragStartSpy.restore();
                this.event = null;
            });

            it('should call dragStart on dragstart event', function () {
                this.view.$('#' + this.model.toJSON().id).trigger(this.event);
                expect(vent.trigger).toHaveBeenCalledWith('drag:start', this.view);
                expect(this.setDataSpy).toHaveBeenCalledWith('Text', 'Sweetie, May I Drag This?');
            });
        });

    });
});
