/*global define, describe, beforeEach, afterEach, waitsFor, it, runs, expect*/
define(['app/views/widgetbar', 'app/collections/widgets'], function (Widgetbar, WidgetCollection) {

    'use strict';

    describe('Views :: Widgetbar.', function () {

        beforeEach(function () {
            this.view = new Widgetbar({
                app: {}
            });
        });

        afterEach(function () {
            this.view.close();
        });

        it('view should be a "ul" element', function () {
            expect(this.view.$el).toBe('ul');
        });

        describe('render()', function () {
            it('should create and render the itemViews', function () {
                this.view.collection = new WidgetCollection([{
                    type: 'text'
                }, {
                    type: 'image'
                }]);

                runs(function () {
                    this.view.render();
                });

                waitsFor(function () {
                    return this.view.children.length === 2;
                }, 750);

                runs(function () {
                    expect(this.view.children.length).toBe(2);
                    expect(this.view.$('li')).toHaveLength(2);
                });

            });
        });

    });

    describe('Views :: Widgetbar with some widgets.', function () {

        beforeEach(function () {
            this.view = new Widgetbar({
                app: {},
                collection: new WidgetCollection([
                    {
                        type: 'image',
                        title: 'Add Image',
                        id: 'add-image',
                        icon: 'picture',
                        description: 'add an image'
                    }, {
                        type: 'text',
                        title: 'Add Text',
                        id: 'add-text',
                        icon: 'font',
                        description: 'add a text'
                    }, {
                        type: 'row',
                        title: 'add Row',
                        id: 'add-row',
                        icon: 'columns',
                        description: 'Add a new row to the page'
                    }
                ])
            });
        });

        it(' should have an image widget with type >image<', function () {
            expect(this.view.collection.at(0).get('type')).toBe('image');
        });

        it(' should have an image widget with id >add-image<', function () {
            expect(this.view.collection.at(0).get('id')).toBe('add-image');
        });

        it(' should have a text widget with type >text<', function () {
            expect(this.view.collection.at(1).get('type')).toBe('text');
        });

        it(' should have a text widget with id >add-text<', function () {
            expect(this.view.collection.at(1).get('id')).toBe('add-text');
        });

        it(' should have a row widget with type >row<', function () {
            expect(this.view.collection.at(2).get('type')).toBe('row');
        });

        it(' should have a row widget with id >add-row<', function () {
            expect(this.view.collection.at(2).get('id')).toBe('add-row');
        });

    });
});
